@@ -1,5 +1,5 @@
 /* tc-310v.h -- Header file for tc-d30v.c.
-   Copyright 1997, 1998, 2000, 2001, 2002, 2005
+   Copyright 1997, 1998, 2000, 2001, 2002, 2005, 2007, 2009, 2011
    Free Software Foundation, Inc.
    Written by Martin Hunt, Cygnus Support.
 
@@ -7,7 +7,7 @@
 
    GAS is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GAS is distributed in the hope that it will be useful,
@@ -22,10 +22,6 @@
 
 #define TC_D30V
 
-#ifndef BFD_ASSEMBLER
- #error D30V support requires BFD_ASSEMBLER
-#endif
-
 /* The target BFD architecture.  */
 #define TARGET_ARCH              bfd_arch_d30v
 #define TARGET_FORMAT            "elf32-d30v"
@@ -50,9 +46,8 @@ extern long md_pcrel_from_section (struct fix *, segT);
 #define md_number_to_chars           number_to_chars_bigendian
 
 int d30v_cleanup (int);
-#define md_after_pass_hook()	     d30v_cleanup (FALSE)
 #define md_cleanup()		     d30v_cleanup (FALSE)
-#define TC_START_LABEL(ch, ptr)      (ch == ':' && d30v_cleanup (FALSE))
+#define TC_START_LABEL(ch, s, ptr)      (ch == ':' && d30v_cleanup (FALSE))
 void d30v_start_line (void);
 #define md_start_line_hook()	     d30v_start_line ()
 
