@@ -1,13 +1,13 @@
 /* tc-i960.h - Basic 80960 instruction formats.
    Copyright 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1997, 1998, 1999,
-   2000, 2001, 2002, 2003
+   2000, 2001, 2002, 2003, 2005, 2007, 2008
    Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
    GAS is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as
-   published by the Free Software Foundation; either version 2,
+   published by the Free Software Foundation; either version 3,
    or (at your option) any later version.
 
    GAS is distributed in the hope that it will be useful, but
@@ -52,21 +52,12 @@
  */
 
 /* tailor gas */
-#define SYMBOLS_NEED_BACKPOINTERS
 #define LOCAL_LABELS_FB 1
 #define BITFIELD_CONS_EXPRESSIONS
 
 /* tailor the coff format */
-#define BFD_ARCH				bfd_arch_i960
-#define COFF_FLAGS				F_AR32WR
 #define COFF_MAGIC				I960ROMAGIC
-#define OBJ_COFF_SECTION_HEADER_HAS_ALIGNMENT
 #define OBJ_COFF_MAX_AUXENTRIES			(2)
-#define TC_COUNT_RELOC(FIX)			(!(FIX)->fx_done)
-#define TC_COFF_FIX2RTYPE(FIX)			tc_coff_fix2rtype (FIX)
-#define TC_COFF_SIZEMACHDEP(FRAGP)		tc_coff_sizemachdep (FRAGP)
-#define TC_COFF_SET_MACHINE(HDRS)		tc_headers_hook (HDRS)
-extern int tc_coff_sizemachdep PARAMS ((struct frag *));
 
 /* MEANING OF 'n_other' in the symbol record.
  *
@@ -140,7 +131,7 @@ struct relocation_info
 /* reloc_callj() may replace a 'call' with a 'calls' or a
    'bal', in which cases it modifies *fixP as appropriate.
    In the case of a 'calls', no further work is required.  */
-extern int reloc_callj PARAMS ((struct fix *));
+extern int reloc_callj (struct fix *);
 
 #define TC_FORCE_RELOCATION_ABS(FIX)		\
   (TC_FORCE_RELOCATION (FIX)			\
@@ -148,7 +139,6 @@ extern int reloc_callj PARAMS ((struct fix *));
 
 #define TC_FORCE_RELOCATION_LOCAL(FIX)		\
   (!(FIX)->fx_pcrel				\
-   || (FIX)->fx_plt				\
    || TC_FORCE_RELOCATION (FIX)		\
    || reloc_callj (FIX))
 
@@ -157,15 +147,9 @@ extern int reloc_callj PARAMS ((struct fix *));
 /* We store the bal information in the sy_tc field.  */
 #define TC_SYMFIELD_TYPE symbolS *
 
-#define TC_ADJUST_RELOC_COUNT(FIX,COUNT) \
-  { fixS *tcfixp = (FIX); \
-    for (;tcfixp;tcfixp=tcfixp->fx_next) \
-      if (tcfixp->fx_tcbit && tcfixp->fx_addsy != 0) \
-	++(COUNT); \
-  }
 #endif
 
-extern int i960_validate_fix PARAMS ((struct fix *, segT));
+extern int i960_validate_fix (struct fix *, segT);
 #define TC_VALIDATE_FIX(FIX,SEGTYPE,LABEL) \
 	if (!i960_validate_fix (FIX, SEGTYPE)) goto LABEL
 
@@ -179,16 +163,15 @@ extern int i960_validate_fix PARAMS ((struct fix *, segT));
 #define MD_APPLY_SYM_VALUE(FIX) 0
 #endif
 
-extern void brtab_emit PARAMS ((void));
+extern void brtab_emit (void);
 #define md_end()	brtab_emit ()
 
-extern void tc_set_bal_of_call PARAMS ((symbolS *, symbolS *));
+extern void tc_set_bal_of_call (symbolS *, symbolS *);
 
-extern struct symbol *tc_get_bal_of_call PARAMS ((symbolS *));
+extern struct symbol *tc_get_bal_of_call (symbolS *);
 
-extern void i960_handle_align PARAMS ((struct frag *));
+extern void i960_handle_align (struct frag *);
 #define HANDLE_ALIGN(FRAG)	i960_handle_align (FRAG)
-#define NEED_FX_R_TYPE
 #define NO_RELOC -1
 
 #define md_operand(x)
