@@ -1,12 +1,12 @@
 /* tc-ns32k.h -- Opcode table for National Semi 32k processor
-   Copyright 1987, 1992, 1993, 1994, 1995, 1997, 2000, 2002, 2005
+   Copyright 1987, 1992, 1993, 1994, 1995, 1997, 2000, 2002, 2005, 2007
    Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
    GAS is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GAS is distributed in the hope that it will be useful,
@@ -26,7 +26,6 @@
 #define TC_PCREL_ADJUST(F) md_pcrel_adjust(F)
 extern int md_pcrel_adjust (fragS *);
 
-#ifdef BFD_ASSEMBLER
 #define NO_RELOC BFD_RELOC_NONE
 
 #define TARGET_ARCH		bfd_arch_ns32k
@@ -34,18 +33,11 @@ extern int md_pcrel_adjust (fragS *);
 #ifndef TARGET_FORMAT		/* Maybe defined in te-*.h.  */
 #define TARGET_FORMAT		"a.out-pc532-mach"
 #endif
-#else
-#define NO_RELOC 0
-#endif
 
 #define LOCAL_LABELS_FB 1
 
 #include "bit_fix.h"
 
-#define tc_aout_pre_write_hook(x)	{;}	/* not used */
-#define tc_crawl_symbol_chain(a)	{;}	/* not used */
-#define tc_headers_hook(a)		{;}	/* not used */
-
 #ifdef SEQUENT_COMPATABILITY
 #define DEF_MODEC 20
 #define DEF_MODEL 21
