@@ -1,6 +1,6 @@
 // merge.h -- handle section merging for gold  -*- C++ -*-
 
-// Copyright 2006, 2007, 2008 Free Software Foundation, Inc.
+// Copyright 2006, 2007, 2008, 2010 Free Software Foundation, Inc.
 // Written by Ian Lance Taylor <iant@google.com>.
 
 // This file is part of gold.
@@ -70,7 +70,7 @@ class Object_merge_map
   bool
   get_output_offset(const Merge_map*, unsigned int shndx,
 		    section_offset_type offset,
-		    section_offset_type *output_offset);
+		    section_offset_type* output_offset);
 
   // Return whether this is the merge map for section SHNDX.
   bool
@@ -152,7 +152,7 @@ class Object_merge_map
   Input_merge_map*
   get_input_merge_map(unsigned int shndx);
 
-  // Get or make the the Input_merge_map to use for the section SHNDX
+  // Get or make the Input_merge_map to use for the section SHNDX
   // with MERGE_MAP.
   Input_merge_map*
   get_or_make_input_merge_map(const Merge_map* merge_map, unsigned int shndx);
@@ -200,7 +200,7 @@ class Merge_map
   bool
   get_output_offset(const Relobj* object, unsigned int shndx,
 		    section_offset_type offset,
-		    section_offset_type *output_offset) const;
+		    section_offset_type* output_offset) const;
 
   // Return whether this is the merge mapping for section SHNDX in
   // OBJECT.  This should return true when get_output_offset would
@@ -216,7 +216,9 @@ class Output_merge_base : public Output_section_data
 {
  public:
   Output_merge_base(uint64_t entsize, uint64_t addralign)
-    : Output_section_data(addralign), merge_map_(), entsize_(entsize)
+    : Output_section_data(addralign), merge_map_(), entsize_(entsize),
+      keeps_input_sections_(false), first_relobj_(NULL), first_shndx_(-1),
+      input_sections_()
   { }
 
   // Return the entry size.
@@ -230,6 +232,52 @@ class Output_merge_base : public Output_section_data
   is_string()
   { return this->do_is_string(); }
 
+  // Whether this keeps input sections.
+  bool
+  keeps_input_sections() const
+  { return this->keeps_input_sections_; }
+
+  // Set the keeps-input-sections flag.  This is virtual so that sub-classes
+  // can perform additional checks.
+  void
+  set_keeps_input_sections()
+  { this->do_set_keeps_input_sections(); }
+
+  // Return the object of the first merged input section.  This used
+  // for script processing.  This is NULL if merge section is empty.
+  Relobj*
+  first_relobj() const
+  { return this->first_relobj_; }
+
+  // Return the section index of the first merged input section.  This
+  // is used for script processing.  This is valid only if merge section
+  // is not valid.
+  unsigned int
+  first_shndx() const
+  { 
+    gold_assert(this->first_relobj_ != NULL);
+    return this->first_shndx_;
+  }
+ 
+  // Set of merged input sections.
+  typedef Unordered_set<Section_id, Section_id_hash> Input_sections;
+
+  // Beginning of merged input sections.
+  Input_sections::const_iterator
+  input_sections_begin() const
+  {
+    gold_assert(this->keeps_input_sections_);
+    return this->input_sections_.begin();
+  }
+
+  // Beginning of merged input sections.
+  Input_sections::const_iterator
+  input_sections_end() const
+  {
+    gold_assert(this->keeps_input_sections_);
+    return this->input_sections_.end();
+  }
+ 
  protected:
   // Return the output offset for an input offset.
   bool
@@ -252,11 +300,20 @@ class Output_merge_base : public Output_section_data
     this->merge_map_.add_mapping(object, shndx, offset, length, output_offset);
   }
 
-  // This may be overriden by the child class.
+  // This may be overridden by the child class.
   virtual bool
   do_is_string()
   { return false; }
 
+  // This may be overridden by the child class.
+  virtual void
+  do_set_keeps_input_sections()
+  { this->keeps_input_sections_ = true; }
+
+  // Record the merged input section for script processing.
+  void
+  record_input_section(Relobj* relobj, unsigned int shndx);
+
  private:
   // A mapping from input object/section/offset to offset in output
   // section.
@@ -264,6 +321,15 @@ class Output_merge_base : public Output_section_data
   // The entry size.  For fixed-size constants, this is the size of
   // the constants.  For strings, this is the size of a character.
   uint64_t entsize_;
+  // Whether we keep input sections.
+  bool keeps_input_sections_;
+  // Object of the first merged input section.  We use this for script
+  // processing.
+  Relobj* first_relobj_;
+  // Section index of the first merged input section. 
+  unsigned int first_shndx_;
+  // Input sections.  We only keep them is keeps_input_sections_ is true.
+  Input_sections input_sections_;
 };
 
 // Handle SHF_MERGE sections with fixed-size constant data.
@@ -303,6 +369,14 @@ class Output_merge_data : public Output_merge_base
   void
   do_print_merge_stats(const char* section_name);
 
+  // Set keeps-input-sections flag.
+  void
+  do_set_keeps_input_sections()
+  {
+    gold_assert(this->input_count_ == 0);
+    Output_merge_base::do_set_keeps_input_sections();
+  }
+
  private:
   // We build a hash table of the fixed-size constants.  Each constant
   // is stored as a pointer into the section data we are accumulating.
@@ -387,10 +461,9 @@ class Output_merge_string : public Output_merge_base
 {
  public:
   Output_merge_string(uint64_t addralign)
-    : Output_merge_base(sizeof(Char_type), addralign), stringpool_(),
-      merged_strings_(), input_count_(0)
+    : Output_merge_base(sizeof(Char_type), addralign), stringpool_(addralign),
+      merged_strings_lists_(), input_count_(0), input_size_(0)
   {
-    gold_assert(addralign <= sizeof(Char_type));
     this->stringpool_.set_no_zero_null();
   }
 
@@ -440,6 +513,14 @@ class Output_merge_string : public Output_merge_base
   do_is_string()
   { return true; }
 
+  // Set keeps-input-sections flag.
+  void
+  do_set_keeps_input_sections()
+  {
+    gold_assert(this->input_count_ == 0);
+    Output_merge_base::do_set_keeps_input_sections();
+  }
+
  private:
   // The name of the string type, for stats.
   const char*
@@ -449,36 +530,43 @@ class Output_merge_string : public Output_merge_base
   // index and offset to strings.
   struct Merged_string
   {
-    // The input object where the string was found.
-    Relobj* object;
-    // The input section in the input object.
-    unsigned int shndx;
     // The offset in the input section.
     section_offset_type offset;
-    // The string itself, a pointer into a Stringpool.
-    const Char_type* string;
-    // The length of the string in bytes, including the null terminator.
-    size_t length;
     // The key in the Stringpool.
     Stringpool::Key stringpool_key;
 
-    Merged_string(Relobj *objecta, unsigned int shndxa,
-		  section_offset_type offseta, const Char_type* stringa,
-		  size_t lengtha, Stringpool::Key stringpool_keya)
-      : object(objecta), shndx(shndxa), offset(offseta), string(stringa),
-	length(lengtha), stringpool_key(stringpool_keya)
+    Merged_string(section_offset_type offseta, Stringpool::Key stringpool_keya)
+      : offset(offseta), stringpool_key(stringpool_keya)
     { }
   };
 
   typedef std::vector<Merged_string> Merged_strings;
 
+  struct Merged_strings_list
+  {
+    // The input object where the strings were found.
+    Relobj* object;
+    // The input section in the input object.
+    unsigned int shndx;
+    // The list of merged strings.
+    Merged_strings merged_strings;
+
+    Merged_strings_list(Relobj* objecta, unsigned int shndxa)
+      : object(objecta), shndx(shndxa), merged_strings()
+    { }
+  };
+
+  typedef std::vector<Merged_strings_list*> Merged_strings_lists;
+
   // As we see the strings, we add them to a Stringpool.
   Stringpool_template<Char_type> stringpool_;
   // Map from a location in an input object to an entry in the
   // Stringpool.
-  Merged_strings merged_strings_;
+  Merged_strings_lists merged_strings_lists_;
   // The number of entries seen in input files.
   size_t input_count_;
+  // The total size of input sections.
+  size_t input_size_;
 };
 
 } // End namespace gold.
