@@ -72,15 +72,14 @@ class Target_test : public Sized_target<size, big_endian>
   { ERROR("call to Target_test::scan_relocatable_relocs"); }
 
   void
-  relocate_for_relocatable(const Relocate_info<size, big_endian>*,
-			   unsigned int, const unsigned char*, size_t,
-                           Output_section*,
-                           typename elfcpp::Elf_types<size>::Elf_Off,
-                           const Relocatable_relocs*, unsigned char*,
-			   typename elfcpp::Elf_types<size>::Elf_Addr,
-			   section_size_type, unsigned char*,
-			   section_size_type)
-  { ERROR("call to Target_test::relocate_for_relocatable"); }
+  relocate_relocs(const Relocate_info<size, big_endian>*,
+		  unsigned int, const unsigned char*, size_t,
+		  Output_section*, typename elfcpp::Elf_types<size>::Elf_Off,
+                  const Relocatable_relocs*, unsigned char*,
+		  typename elfcpp::Elf_types<size>::Elf_Addr,
+		  section_size_type, unsigned char*,
+		  section_size_type)
+  { ERROR("call to Target_test::relocate_relocs"); }
 
   static const Target::Target_info test_target_info;
 };
@@ -108,7 +107,8 @@ const Target::Target_info Target_test<size, big_endian>::test_target_info =
   0,					// small_common_section_flags
   0,					// large_common_section_flags
   NULL,					// attributes_section
-  NULL					// attributes_vendor
+  NULL,					// attributes_vendor
+  "_start"		// entry_symbol_name
 };
 
 // The test targets.
