@@ -82,7 +82,7 @@ typedef struct name_list {
 name_list;
 
 typedef enum {sort_none, sort_ascending, sort_descending} sort_order;
-  
+
 /* A wildcard specification.  */
 
 typedef enum {
@@ -274,6 +274,9 @@ typedef struct {
      numbers everywhere.  */
   bfd_boolean sane_expr;
 
+  /* If set, code and non-code sections should never be in one segment.  */
+  bfd_boolean separate_code;
+
   /* The rpath separation character.  Usually ':'.  */
   char rpath_separator;
 
