@@ -78,88 +78,95 @@
      DATA_SEGMENT_END = 294,
      SORT_BY_NAME = 295,
      SORT_BY_ALIGNMENT = 296,
-     SIZEOF_HEADERS = 297,
-     OUTPUT_FORMAT = 298,
-     FORCE_COMMON_ALLOCATION = 299,
-     OUTPUT_ARCH = 300,
-     INHIBIT_COMMON_ALLOCATION = 301,
-     SEGMENT_START = 302,
-     INCLUDE = 303,
-     MEMORY = 304,
-     REGION_ALIAS = 305,
-     NOLOAD = 306,
-     DSECT = 307,
-     COPY = 308,
-     INFO = 309,
-     OVERLAY = 310,
-     DEFINED = 311,
-     TARGET_K = 312,
-     SEARCH_DIR = 313,
-     MAP = 314,
-     ENTRY = 315,
-     NEXT = 316,
-     SIZEOF = 317,
-     ALIGNOF = 318,
-     ADDR = 319,
-     LOADADDR = 320,
-     MAX_K = 321,
-     MIN_K = 322,
-     STARTUP = 323,
-     HLL = 324,
-     SYSLIB = 325,
-     FLOAT = 326,
-     NOFLOAT = 327,
-     NOCROSSREFS = 328,
-     ORIGIN = 329,
-     FILL = 330,
-     LENGTH = 331,
-     CREATE_OBJECT_SYMBOLS = 332,
-     INPUT = 333,
-     GROUP = 334,
-     OUTPUT = 335,
-     CONSTRUCTORS = 336,
-     ALIGNMOD = 337,
-     AT = 338,
-     SUBALIGN = 339,
-     PROVIDE = 340,
-     PROVIDE_HIDDEN = 341,
-     AS_NEEDED = 342,
-     CHIP = 343,
-     LIST = 344,
-     SECT = 345,
-     ABSOLUTE = 346,
-     LOAD = 347,
-     NEWLINE = 348,
-     ENDWORD = 349,
-     ORDER = 350,
-     NAMEWORD = 351,
-     ASSERT_K = 352,
-     FORMAT = 353,
-     PUBLIC = 354,
-     DEFSYMEND = 355,
-     BASE = 356,
-     ALIAS = 357,
-     TRUNCATE = 358,
-     REL = 359,
-     INPUT_SCRIPT = 360,
-     INPUT_MRI_SCRIPT = 361,
-     INPUT_DEFSYM = 362,
-     CASE = 363,
-     EXTERN = 364,
-     START = 365,
-     VERS_TAG = 366,
-     VERS_IDENTIFIER = 367,
-     GLOBAL = 368,
-     LOCAL = 369,
-     VERSIONK = 370,
-     INPUT_VERSION_SCRIPT = 371,
-     KEEP = 372,
-     ONLY_IF_RO = 373,
-     ONLY_IF_RW = 374,
-     SPECIAL = 375,
-     EXCLUDE_FILE = 376,
-     CONSTANT = 377,
-     INPUT_DYNAMIC_LIST = 378
+     SORT_NONE = 297,
+     SORT_BY_INIT_PRIORITY = 298,
+     SIZEOF_HEADERS = 299,
+     OUTPUT_FORMAT = 300,
+     FORCE_COMMON_ALLOCATION = 301,
+     OUTPUT_ARCH = 302,
+     INHIBIT_COMMON_ALLOCATION = 303,
+     SEGMENT_START = 304,
+     INCLUDE = 305,
+     MEMORY = 306,
+     REGION_ALIAS = 307,
+     LD_FEATURE = 308,
+     NOLOAD = 309,
+     DSECT = 310,
+     COPY = 311,
+     INFO = 312,
+     OVERLAY = 313,
+     DEFINED = 314,
+     TARGET_K = 315,
+     SEARCH_DIR = 316,
+     MAP = 317,
+     ENTRY = 318,
+     NEXT = 319,
+     SIZEOF = 320,
+     ALIGNOF = 321,
+     ADDR = 322,
+     LOADADDR = 323,
+     MAX_K = 324,
+     MIN_K = 325,
+     STARTUP = 326,
+     HLL = 327,
+     SYSLIB = 328,
+     FLOAT = 329,
+     NOFLOAT = 330,
+     NOCROSSREFS = 331,
+     ORIGIN = 332,
+     FILL = 333,
+     LENGTH = 334,
+     CREATE_OBJECT_SYMBOLS = 335,
+     INPUT = 336,
+     GROUP = 337,
+     OUTPUT = 338,
+     CONSTRUCTORS = 339,
+     ALIGNMOD = 340,
+     AT = 341,
+     SUBALIGN = 342,
+     HIDDEN = 343,
+     PROVIDE = 344,
+     PROVIDE_HIDDEN = 345,
+     AS_NEEDED = 346,
+     CHIP = 347,
+     LIST = 348,
+     SECT = 349,
+     ABSOLUTE = 350,
+     LOAD = 351,
+     NEWLINE = 352,
+     ENDWORD = 353,
+     ORDER = 354,
+     NAMEWORD = 355,
+     ASSERT_K = 356,
+     LOG2CEIL = 357,
+     FORMAT = 358,
+     PUBLIC = 359,
+     DEFSYMEND = 360,
+     BASE = 361,
+     ALIAS = 362,
+     TRUNCATE = 363,
+     REL = 364,
+     INPUT_SCRIPT = 365,
+     INPUT_MRI_SCRIPT = 366,
+     INPUT_DEFSYM = 367,
+     CASE = 368,
+     EXTERN = 369,
+     START = 370,
+     VERS_TAG = 371,
+     VERS_IDENTIFIER = 372,
+     GLOBAL = 373,
+     LOCAL = 374,
+     VERSIONK = 375,
+     INPUT_VERSION_SCRIPT = 376,
+     KEEP = 377,
+     ONLY_IF_RO = 378,
+     ONLY_IF_RW = 379,
+     SPECIAL = 380,
+     INPUT_SECTION_FLAGS = 381,
+     ALIGN_WITH_INPUT = 382,
+     EXCLUDE_FILE = 383,
+     CONSTANT = 384,
+     INPUT_DYNAMIC_LIST = 385
    };
 #endif
 /* Tokens.  */
@@ -202,95 +209,102 @@
 #define DATA_SEGMENT_END 294
 #define SORT_BY_NAME 295
 #define SORT_BY_ALIGNMENT 296
-#define SIZEOF_HEADERS 297
-#define OUTPUT_FORMAT 298
-#define FORCE_COMMON_ALLOCATION 299
-#define OUTPUT_ARCH 300
-#define INHIBIT_COMMON_ALLOCATION 301
-#define SEGMENT_START 302
-#define INCLUDE 303
-#define MEMORY 304
-#define REGION_ALIAS 305
-#define NOLOAD 306
-#define DSECT 307
-#define COPY 308
-#define INFO 309
-#define OVERLAY 310
-#define DEFINED 311
-#define TARGET_K 312
-#define SEARCH_DIR 313
-#define MAP 314
-#define ENTRY 315
-#define NEXT 316
-#define SIZEOF 317
-#define ALIGNOF 318
-#define ADDR 319
-#define LOADADDR 320
-#define MAX_K 321
-#define MIN_K 322
-#define STARTUP 323
-#define HLL 324
-#define SYSLIB 325
-#define FLOAT 326
-#define NOFLOAT 327
-#define NOCROSSREFS 328
-#define ORIGIN 329
-#define FILL 330
-#define LENGTH 331
-#define CREATE_OBJECT_SYMBOLS 332
-#define INPUT 333
-#define GROUP 334
-#define OUTPUT 335
-#define CONSTRUCTORS 336
-#define ALIGNMOD 337
-#define AT 338
-#define SUBALIGN 339
-#define PROVIDE 340
-#define PROVIDE_HIDDEN 341
-#define AS_NEEDED 342
-#define CHIP 343
-#define LIST 344
-#define SECT 345
-#define ABSOLUTE 346
-#define LOAD 347
-#define NEWLINE 348
-#define ENDWORD 349
-#define ORDER 350
-#define NAMEWORD 351
-#define ASSERT_K 352
-#define FORMAT 353
-#define PUBLIC 354
-#define DEFSYMEND 355
-#define BASE 356
-#define ALIAS 357
-#define TRUNCATE 358
-#define REL 359
-#define INPUT_SCRIPT 360
-#define INPUT_MRI_SCRIPT 361
-#define INPUT_DEFSYM 362
-#define CASE 363
-#define EXTERN 364
-#define START 365
-#define VERS_TAG 366
-#define VERS_IDENTIFIER 367
-#define GLOBAL 368
-#define LOCAL 369
-#define VERSIONK 370
-#define INPUT_VERSION_SCRIPT 371
-#define KEEP 372
-#define ONLY_IF_RO 373
-#define ONLY_IF_RW 374
-#define SPECIAL 375
-#define EXCLUDE_FILE 376
-#define CONSTANT 377
-#define INPUT_DYNAMIC_LIST 378
+#define SORT_NONE 297
+#define SORT_BY_INIT_PRIORITY 298
+#define SIZEOF_HEADERS 299
+#define OUTPUT_FORMAT 300
+#define FORCE_COMMON_ALLOCATION 301
+#define OUTPUT_ARCH 302
+#define INHIBIT_COMMON_ALLOCATION 303
+#define SEGMENT_START 304
+#define INCLUDE 305
+#define MEMORY 306
+#define REGION_ALIAS 307
+#define LD_FEATURE 308
+#define NOLOAD 309
+#define DSECT 310
+#define COPY 311
+#define INFO 312
+#define OVERLAY 313
+#define DEFINED 314
+#define TARGET_K 315
+#define SEARCH_DIR 316
+#define MAP 317
+#define ENTRY 318
+#define NEXT 319
+#define SIZEOF 320
+#define ALIGNOF 321
+#define ADDR 322
+#define LOADADDR 323
+#define MAX_K 324
+#define MIN_K 325
+#define STARTUP 326
+#define HLL 327
+#define SYSLIB 328
+#define FLOAT 329
+#define NOFLOAT 330
+#define NOCROSSREFS 331
+#define ORIGIN 332
+#define FILL 333
+#define LENGTH 334
+#define CREATE_OBJECT_SYMBOLS 335
+#define INPUT 336
+#define GROUP 337
+#define OUTPUT 338
+#define CONSTRUCTORS 339
+#define ALIGNMOD 340
+#define AT 341
+#define SUBALIGN 342
+#define HIDDEN 343
+#define PROVIDE 344
+#define PROVIDE_HIDDEN 345
+#define AS_NEEDED 346
+#define CHIP 347
+#define LIST 348
+#define SECT 349
+#define ABSOLUTE 350
+#define LOAD 351
+#define NEWLINE 352
+#define ENDWORD 353
+#define ORDER 354
+#define NAMEWORD 355
+#define ASSERT_K 356
+#define LOG2CEIL 357
+#define FORMAT 358
+#define PUBLIC 359
+#define DEFSYMEND 360
+#define BASE 361
+#define ALIAS 362
+#define TRUNCATE 363
+#define REL 364
+#define INPUT_SCRIPT 365
+#define INPUT_MRI_SCRIPT 366
+#define INPUT_DEFSYM 367
+#define CASE 368
+#define EXTERN 369
+#define START 370
+#define VERS_TAG 371
+#define VERS_IDENTIFIER 372
+#define GLOBAL 373
+#define LOCAL 374
+#define VERSIONK 375
+#define INPUT_VERSION_SCRIPT 376
+#define KEEP 377
+#define ONLY_IF_RO 378
+#define ONLY_IF_RW 379
+#define SPECIAL 380
+#define INPUT_SECTION_FLAGS 381
+#define ALIGN_WITH_INPUT 382
+#define EXCLUDE_FILE 383
+#define CONSTANT 384
+#define INPUT_DYNAMIC_LIST 385
 
 
 
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 62 "ldgram.y"
+#line 60 "ldgram.y"
 {
   bfd_vma integer;
   struct big_int
@@ -304,6 +318,8 @@ typedef union YYSTYPE
   struct wildcard_spec wildcard;
   struct wildcard_list *wildcard_list;
   struct name_list *name_list;
+  struct flag_info_list *flag_info_list;
+  struct flag_info *flag_info;
   int token;
   union etree_union *etree;
   struct phdr_info
@@ -320,7 +336,7 @@ typedef union YYSTYPE
   struct bfd_elf_version_tree *versnode;
 }
 /* Line 1529 of yacc.c.  */
-#line 324 "ldgram.h"
+#line 340 "ldgram.h"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
