@@ -1,5 +1,5 @@
 /* Plugin control for the GNU linker.
-   Copyright 2010, 2011, 2012 Free Software Foundation, Inc.
+   Copyright 2010, 2011, 2012, 2013 Free Software Foundation, Inc.
 
    This file is part of the GNU Binutils.
 
@@ -39,9 +39,6 @@
 /* Report plugin symbols.  */
 bfd_boolean report_plugin_symbols;
 
-/* Store plugin intermediate files permanently.  */
-bfd_boolean plugin_save_temps;
-
 /* The suffix to append to the name of the real (claimed) object file
    when generating a dummy BFD to hold the IR symbols sent from the
    plugin.  For cosmetic use only; appears in maps, crefs etc.  */
@@ -220,17 +217,6 @@ plugin_opt_plugin_arg (const char *arg)
   if (!last_plugin)
     return set_plugin_error (_("<no plugin>"));
 
-  /* Ignore -pass-through= from GCC driver.  */
-  if (*arg == '-')
-    {
-      const char *p;
-      for (p = arg + 1; p; p++)
-	if (*p != '-')
-	  break;
-      if (strncmp (p, "pass-through=", 13) == 0)
-	return 0;
-    }
-
   newarg = xmalloc (sizeof *newarg);
   newarg->arg = arg;
   newarg->next = NULL;
@@ -443,20 +429,18 @@ add_symbols (void *handle, int nsyms, const struct ld_plugin_symbol *syms)
 /* Get the input file information with an open (possibly re-opened)
    file descriptor.  */
 static enum ld_plugin_status
-get_input_file (const void *handle, struct ld_plugin_input_file *file)
+get_input_file (const void *handle ATTRIBUTE_UNUSED,
+                struct ld_plugin_input_file *file ATTRIBUTE_UNUSED)
 {
   ASSERT (called_plugin);
-  handle = handle;
-  file = file;
   return LDPS_ERR;
 }
 
 /* Release the input file.  */
 static enum ld_plugin_status
-release_input_file (const void *handle)
+release_input_file (const void *handle ATTRIBUTE_UNUSED)
 {
   ASSERT (called_plugin);
-  handle = handle;
   return LDPS_ERR;
 }
 
@@ -890,9 +874,6 @@ plugin_maybe_claim (struct ld_plugin_input_file *file,
   close (file->fd);
   if (claimed)
     {
-      /* Check object only section.  */
-      cmdline_check_object_only_section (entry->the_bfd, TRUE);
-
       /* Discard the real file's BFD and substitute the dummy one.  */
 
       /* BFD archive handling caches elements so we can't call
@@ -946,17 +927,14 @@ plugin_call_cleanup (void)
     {
       if (curplug->cleanup_handler && !curplug->cleanup_done)
 	{
-	  if (!plugin_save_temps)
-	    {
-	      enum ld_plugin_status rv;
-	      curplug->cleanup_done = TRUE;
-	      called_plugin = curplug;
-	      rv = (*curplug->cleanup_handler) ();
-	      called_plugin = NULL;
-	      if (rv != LDPS_OK)
-		info_msg (_("%P: %s: error in plugin cleanup: %d (ignored)\n"),
-			  curplug->name, rv);
-	    }
+	  enum ld_plugin_status rv;
+	  curplug->cleanup_done = TRUE;
+	  called_plugin = curplug;
+	  rv = (*curplug->cleanup_handler) ();
+	  called_plugin = NULL;
+	  if (rv != LDPS_OK)
+	    info_msg (_("%P: %s: error in plugin cleanup: %d (ignored)\n"),
+		      curplug->name, rv);
 	  dlclose (curplug->dlhandle);
 	}
       curplug = curplug->next;
@@ -982,14 +960,14 @@ plugin_notice (struct bfd_link_info *info,
     {
       bfd *sym_bfd;
 
-      /* No further processing if this def/ref is from an IR dummy BFD.  */
+      /* Nothing to do here if this def/ref is from an IR dummy BFD.  */
       if (is_ir_dummy_bfd (abfd))
-	return TRUE;
+	;
 
       /* Making an indirect symbol counts as a reference unless this
 	 is a brand new symbol.  */
-      if (bfd_is_ind_section (section)
-	  || (flags & BSF_INDIRECT) != 0)
+      else if (bfd_is_ind_section (section)
+	       || (flags & BSF_INDIRECT) != 0)
 	{
 	  if (h->type != bfd_link_hash_new)
 	    {
@@ -1013,7 +991,15 @@ plugin_notice (struct bfd_link_info *info,
 
       /* If this is a ref, set non_ir_ref.  */
       else if (bfd_is_und_section (section))
-	h->non_ir_ref = TRUE;
+	{
+	  /* Replace the undefined dummy bfd with the real one.  */
+	  if ((h->type == bfd_link_hash_undefined
+	       || h->type == bfd_link_hash_undefweak)
+	      && (h->u.undef.abfd == NULL
+		  || (h->u.undef.abfd->flags & BFD_PLUGIN) != 0))
+	    h->u.undef.abfd = abfd;
+	  h->non_ir_ref = TRUE;
+	}
 
       /* Otherwise, it must be a new def.  Ensure any symbol defined
 	 in an IR dummy BFD takes on a new value from a real BFD.
